"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const overmind_1 = require("overmind");
const testRender_1 = require("./testRender");
const _1 = require("./");
const hooks_1 = require("./hooks");
describe('HOOKS', () => {
    describe('STATE', () => {
        test('should render component with initial state', () => {
            const app = new overmind_1.Overmind({});
            const TestComponent = () => {
                const [state] = hooks_1.useState('foo');
                return _1.h("div", null, state);
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const target = test.getTargetElement();
            expect(target.children[0].vtree.children[0].el.nodeValue).toBe('foo');
        });
        test('should rerender component with new state', () => {
            const app = new overmind_1.Overmind({});
            const TestComponent = () => {
                const [state] = hooks_1.useState('foo');
                return _1.h("div", null, state);
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const target = test.getTargetElement();
            target.children[0].hooks[0][1]('bar');
            expect(target.children[0].vtree.children[0].el.nodeValue).toBe('bar');
        });
    });
    describe('EFFECT', () => {
        test('should run effect on initial render', () => {
            const app = new overmind_1.Overmind({});
            let hasRun = false;
            const TestComponent = () => {
                hooks_1.useEffect(() => {
                    hasRun = true;
                });
                return null;
            };
            testRender_1.testRender(app, _1.h(TestComponent, null));
            expect(hasRun).toBe(true);
        });
        test('should rerun effect on new renders', () => {
            const app = new overmind_1.Overmind({});
            let runCount = 0;
            const TestComponent = () => {
                hooks_1.useEffect(() => {
                    runCount++;
                });
                return null;
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const component = test.getTargetElement().children[0];
            component.reconcile(component, true);
            expect(runCount).toBe(2);
        });
        test('should not rerun effect with empty array as second arg', () => {
            const app = new overmind_1.Overmind({});
            let runCount = 0;
            const TestComponent = () => {
                hooks_1.useEffect(() => {
                    runCount++;
                }, []);
                return null;
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const component = test.getTargetElement().children[0];
            component.reconcile(component, true);
            expect(runCount).toBe(1);
        });
        test('should not rerun effect with matching array', () => {
            const app = new overmind_1.Overmind({});
            let runCount = 0;
            const TestComponent = () => {
                hooks_1.useEffect(() => {
                    runCount++;
                }, ['foo']);
                return null;
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const component = test.getTargetElement().children[0];
            component.reconcile(component, true);
            expect(runCount).toBe(1);
        });
        test('should rerun effect with non matching array', () => {
            const app = new overmind_1.Overmind({});
            let runCount = 0;
            const TestComponent = () => {
                hooks_1.useEffect(() => {
                    runCount++;
                }, [runCount]);
                return null;
            };
            const test = testRender_1.testRender(app, _1.h(TestComponent, null));
            const component = test.getTargetElement().children[0];
            component.reconcile(component, true);
            expect(runCount).toBe(2);
        });
    });
    describe('REF', () => {
        test.only('should provide element ref', () => {
            const app = new overmind_1.Overmind({});
            let providedRef;
            const TestComponent = () => {
                const ref = hooks_1.useRef();
                providedRef = ref;
                return _1.h("div", { ref: ref });
            };
            testRender_1.testRender(app, _1.h(TestComponent, null));
            expect(providedRef.current.tag).toBe('div');
        });
    });
});
//# sourceMappingURL=hooks.test.js.map